/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 * Copyright (C) 2013, 2014 Apple Inc. All rights reserved.
 * Copyright (C) 2014 University of Washington. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

// DO NOT EDIT THIS FILE. It is automatically generated from InspectorJS.json
// by the script: Source/JavaScriptCore/inspector/scripts/generate-inspector-protocol-bindings.py

#ifndef InspectorJSBackendDispatchers_h
#define InspectorJSBackendDispatchers_h

#if ENABLE(INSPECTOR)

#include "InspectorJSProtocolTypes.h"
#include <inspector/InspectorBackendDispatcher.h>
#include <wtf/PassRefPtr.h>
#include <wtf/text/WTFString.h>

namespace Inspector {

typedef String ErrorString;

class JS_EXPORT_PRIVATE InspectorConsoleBackendDispatcherHandler {
public:
    virtual void enable(ErrorString*) = 0;
    virtual void disable(ErrorString*) = 0;
    virtual void clearMessages(ErrorString*) = 0;
    virtual void setMonitoringXHREnabled(ErrorString*, bool in_enabled) = 0;
    virtual void addInspectedNode(ErrorString*, int in_nodeId) = 0;
protected:
    virtual ~InspectorConsoleBackendDispatcherHandler();
};

class JS_EXPORT_PRIVATE InspectorDebuggerBackendDispatcherHandler {
public:
    virtual void enable(ErrorString*) = 0;
    virtual void disable(ErrorString*) = 0;
    virtual void setBreakpointsActive(ErrorString*, bool in_active) = 0;
    virtual void setBreakpointByUrl(ErrorString*, int in_lineNumber, const String* in_url, const String* in_urlRegex, const int* in_columnNumber, const RefPtr<Inspector::InspectorObject>* in_options, String* out_breakpointId, RefPtr<Inspector::Protocol::Array<Inspector::Protocol::Debugger::Location>>& out_locations) = 0;
    virtual void setBreakpoint(ErrorString*, const RefPtr<Inspector::InspectorObject>& in_location, const RefPtr<Inspector::InspectorObject>* in_options, String* out_breakpointId, RefPtr<Inspector::Protocol::Debugger::Location>& out_actualLocation) = 0;
    virtual void removeBreakpoint(ErrorString*, const String& in_breakpointId) = 0;
    virtual void continueToLocation(ErrorString*, const RefPtr<Inspector::InspectorObject>& in_location) = 0;
    virtual void stepOver(ErrorString*) = 0;
    virtual void stepInto(ErrorString*) = 0;
    virtual void stepOut(ErrorString*) = 0;
    virtual void pause(ErrorString*) = 0;
    virtual void resume(ErrorString*) = 0;
    virtual void searchInContent(ErrorString*, const String& in_scriptId, const String& in_query, const bool* in_caseSensitive, const bool* in_isRegex, RefPtr<Inspector::Protocol::Array<Inspector::Protocol::GenericTypes::SearchMatch>>& out_result) = 0;
    virtual void getScriptSource(ErrorString*, const String& in_scriptId, String* out_scriptSource) = 0;
    virtual void getFunctionDetails(ErrorString*, const String& in_functionId, RefPtr<Inspector::Protocol::Debugger::FunctionDetails>& out_details) = 0;
    // Named after parameter 'state' while generating command/event setPauseOnExceptions.
    enum class State {
        None = 45,
        Uncaught = 54,
        All = 55,
    }; // enum class State
    virtual void setPauseOnExceptions(ErrorString*, const String& in_state) = 0;
    virtual void evaluateOnCallFrame(ErrorString*, const String& in_callFrameId, const String& in_expression, const String* in_objectGroup, const bool* in_includeCommandLineAPI, const bool* in_doNotPauseOnExceptionsAndMuteConsole, const bool* in_returnByValue, const bool* in_generatePreview, RefPtr<Inspector::Protocol::Runtime::RemoteObject>& out_result, Inspector::Protocol::OptOutput<bool>* opt_out_wasThrown) = 0;
    virtual void setOverlayMessage(ErrorString*, const String* in_message) = 0;
protected:
    virtual ~InspectorDebuggerBackendDispatcherHandler();
};

class JS_EXPORT_PRIVATE InspectorInspectorBackendDispatcherHandler {
public:
    virtual void enable(ErrorString*) = 0;
    virtual void disable(ErrorString*) = 0;
protected:
    virtual ~InspectorInspectorBackendDispatcherHandler();
};

class JS_EXPORT_PRIVATE InspectorRuntimeBackendDispatcherHandler {
public:
    // Named after parameter 'result' while generating command/event parse.
    enum class Result {
        None = 45,
        Irrecoverable = 46,
        UnterminatedLiteral = 47,
        Recoverable = 48,
    }; // enum class Result
    virtual void parse(ErrorString*, const String& in_source, Inspector::Protocol::Runtime::SyntaxErrorType* out_result, Inspector::Protocol::OptOutput<String>* opt_out_message, RefPtr<Inspector::Protocol::Runtime::ErrorRange>& opt_out_range) = 0;
    virtual void evaluate(ErrorString*, const String& in_expression, const String* in_objectGroup, const bool* in_includeCommandLineAPI, const bool* in_doNotPauseOnExceptionsAndMuteConsole, const int* in_contextId, const bool* in_returnByValue, const bool* in_generatePreview, RefPtr<Inspector::Protocol::Runtime::RemoteObject>& out_result, Inspector::Protocol::OptOutput<bool>* opt_out_wasThrown) = 0;
    virtual void callFunctionOn(ErrorString*, const String& in_objectId, const String& in_functionDeclaration, const RefPtr<Inspector::InspectorArray>* in_arguments, const bool* in_doNotPauseOnExceptionsAndMuteConsole, const bool* in_returnByValue, const bool* in_generatePreview, RefPtr<Inspector::Protocol::Runtime::RemoteObject>& out_result, Inspector::Protocol::OptOutput<bool>* opt_out_wasThrown) = 0;
    virtual void getProperties(ErrorString*, const String& in_objectId, const bool* in_ownProperties, RefPtr<Inspector::Protocol::Array<Inspector::Protocol::Runtime::PropertyDescriptor>>& out_result, RefPtr<Inspector::Protocol::Array<Inspector::Protocol::Runtime::InternalPropertyDescriptor>>& opt_out_internalProperties) = 0;
    virtual void releaseObject(ErrorString*, const String& in_objectId) = 0;
    virtual void releaseObjectGroup(ErrorString*, const String& in_objectGroup) = 0;
    virtual void run(ErrorString*) = 0;
    virtual void enable(ErrorString*) = 0;
    virtual void disable(ErrorString*) = 0;
    virtual void getRuntimeTypesForVariablesAtOffsets(ErrorString*, const RefPtr<Inspector::InspectorArray>& in_locations, RefPtr<Inspector::Protocol::Array<Inspector::Protocol::Runtime::TypeDescription>>& out_types) = 0;
    virtual void enableTypeProfiler(ErrorString*) = 0;
    virtual void disableTypeProfiler(ErrorString*) = 0;
protected:
    virtual ~InspectorRuntimeBackendDispatcherHandler();
};

class JS_EXPORT_PRIVATE InspectorConsoleBackendDispatcher final : public Inspector::InspectorSupplementalBackendDispatcher {
public:
    static PassRefPtr<InspectorConsoleBackendDispatcher> create(Inspector::InspectorBackendDispatcher*, InspectorConsoleBackendDispatcherHandler*);
    virtual void dispatch(long callId, const String& method, PassRefPtr<Inspector::InspectorObject> message) override;
private:
    void enable(long callId, const Inspector::InspectorObject& message);
    void disable(long callId, const Inspector::InspectorObject& message);
    void clearMessages(long callId, const Inspector::InspectorObject& message);
    void setMonitoringXHREnabled(long callId, const Inspector::InspectorObject& message);
    void addInspectedNode(long callId, const Inspector::InspectorObject& message);
private:
    InspectorConsoleBackendDispatcher(Inspector::InspectorBackendDispatcher*, InspectorConsoleBackendDispatcherHandler*);
    InspectorConsoleBackendDispatcherHandler* m_agent;
};

class JS_EXPORT_PRIVATE InspectorDebuggerBackendDispatcher final : public Inspector::InspectorSupplementalBackendDispatcher {
public:
    static PassRefPtr<InspectorDebuggerBackendDispatcher> create(Inspector::InspectorBackendDispatcher*, InspectorDebuggerBackendDispatcherHandler*);
    virtual void dispatch(long callId, const String& method, PassRefPtr<Inspector::InspectorObject> message) override;
private:
    void enable(long callId, const Inspector::InspectorObject& message);
    void disable(long callId, const Inspector::InspectorObject& message);
    void setBreakpointsActive(long callId, const Inspector::InspectorObject& message);
    void setBreakpointByUrl(long callId, const Inspector::InspectorObject& message);
    void setBreakpoint(long callId, const Inspector::InspectorObject& message);
    void removeBreakpoint(long callId, const Inspector::InspectorObject& message);
    void continueToLocation(long callId, const Inspector::InspectorObject& message);
    void stepOver(long callId, const Inspector::InspectorObject& message);
    void stepInto(long callId, const Inspector::InspectorObject& message);
    void stepOut(long callId, const Inspector::InspectorObject& message);
    void pause(long callId, const Inspector::InspectorObject& message);
    void resume(long callId, const Inspector::InspectorObject& message);
    void searchInContent(long callId, const Inspector::InspectorObject& message);
    void getScriptSource(long callId, const Inspector::InspectorObject& message);
    void getFunctionDetails(long callId, const Inspector::InspectorObject& message);
    void setPauseOnExceptions(long callId, const Inspector::InspectorObject& message);
    void evaluateOnCallFrame(long callId, const Inspector::InspectorObject& message);
    void setOverlayMessage(long callId, const Inspector::InspectorObject& message);
private:
    InspectorDebuggerBackendDispatcher(Inspector::InspectorBackendDispatcher*, InspectorDebuggerBackendDispatcherHandler*);
    InspectorDebuggerBackendDispatcherHandler* m_agent;
};

class JS_EXPORT_PRIVATE InspectorInspectorBackendDispatcher final : public Inspector::InspectorSupplementalBackendDispatcher {
public:
    static PassRefPtr<InspectorInspectorBackendDispatcher> create(Inspector::InspectorBackendDispatcher*, InspectorInspectorBackendDispatcherHandler*);
    virtual void dispatch(long callId, const String& method, PassRefPtr<Inspector::InspectorObject> message) override;
private:
    void enable(long callId, const Inspector::InspectorObject& message);
    void disable(long callId, const Inspector::InspectorObject& message);
private:
    InspectorInspectorBackendDispatcher(Inspector::InspectorBackendDispatcher*, InspectorInspectorBackendDispatcherHandler*);
    InspectorInspectorBackendDispatcherHandler* m_agent;
};

class JS_EXPORT_PRIVATE InspectorRuntimeBackendDispatcher final : public Inspector::InspectorSupplementalBackendDispatcher {
public:
    static PassRefPtr<InspectorRuntimeBackendDispatcher> create(Inspector::InspectorBackendDispatcher*, InspectorRuntimeBackendDispatcherHandler*);
    virtual void dispatch(long callId, const String& method, PassRefPtr<Inspector::InspectorObject> message) override;
private:
    void parse(long callId, const Inspector::InspectorObject& message);
    void evaluate(long callId, const Inspector::InspectorObject& message);
    void callFunctionOn(long callId, const Inspector::InspectorObject& message);
    void getProperties(long callId, const Inspector::InspectorObject& message);
    void releaseObject(long callId, const Inspector::InspectorObject& message);
    void releaseObjectGroup(long callId, const Inspector::InspectorObject& message);
    void run(long callId, const Inspector::InspectorObject& message);
    void enable(long callId, const Inspector::InspectorObject& message);
    void disable(long callId, const Inspector::InspectorObject& message);
    void getRuntimeTypesForVariablesAtOffsets(long callId, const Inspector::InspectorObject& message);
    void enableTypeProfiler(long callId, const Inspector::InspectorObject& message);
    void disableTypeProfiler(long callId, const Inspector::InspectorObject& message);
private:
    InspectorRuntimeBackendDispatcher(Inspector::InspectorBackendDispatcher*, InspectorRuntimeBackendDispatcherHandler*);
    InspectorRuntimeBackendDispatcherHandler* m_agent;
};

} // namespace Inspector

#endif // ENABLE(INSPECTOR)

#endif // !defined(InspectorJSBackendDispatchers_h)
