/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 * Copyright (C) 2013, 2014 Apple Inc. All rights reserved.
 * Copyright (C) 2014 University of Washington. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

// DO NOT EDIT THIS FILE. It is automatically generated from InspectorWeb.json
// by the script: Source/JavaScriptCore/inspector/scripts/generate-inspector-protocol-bindings.py

#include "config.h"
#include "InspectorWebBackendDispatchers.h"

#if ENABLE(INSPECTOR)

#include <inspector/InspectorFrontendChannel.h>
#include <inspector/InspectorValues.h>
#include <wtf/text/CString.h>

namespace Inspector {

InspectorApplicationCacheBackendDispatcherHandler::~InspectorApplicationCacheBackendDispatcherHandler() { }
InspectorCSSBackendDispatcherHandler::~InspectorCSSBackendDispatcherHandler() { }
InspectorDOMBackendDispatcherHandler::~InspectorDOMBackendDispatcherHandler() { }
InspectorDOMDebuggerBackendDispatcherHandler::~InspectorDOMDebuggerBackendDispatcherHandler() { }
InspectorDOMStorageBackendDispatcherHandler::~InspectorDOMStorageBackendDispatcherHandler() { }
#if ENABLE(SQL_DATABASE)
InspectorDatabaseBackendDispatcherHandler::~InspectorDatabaseBackendDispatcherHandler() { }
#endif // ENABLE(SQL_DATABASE)
#if ENABLE(INDEXED_DATABASE)
InspectorIndexedDBBackendDispatcherHandler::~InspectorIndexedDBBackendDispatcherHandler() { }
#endif // ENABLE(INDEXED_DATABASE)
InspectorLayerTreeBackendDispatcherHandler::~InspectorLayerTreeBackendDispatcherHandler() { }
InspectorNetworkBackendDispatcherHandler::~InspectorNetworkBackendDispatcherHandler() { }
InspectorPageBackendDispatcherHandler::~InspectorPageBackendDispatcherHandler() { }
InspectorTimelineBackendDispatcherHandler::~InspectorTimelineBackendDispatcherHandler() { }
InspectorWorkerBackendDispatcherHandler::~InspectorWorkerBackendDispatcherHandler() { }

PassRefPtr<InspectorApplicationCacheBackendDispatcher> InspectorApplicationCacheBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorApplicationCacheBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorApplicationCacheBackendDispatcher(backendDispatcher, agent));
}

InspectorApplicationCacheBackendDispatcher::InspectorApplicationCacheBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorApplicationCacheBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("ApplicationCache"), this);
}

void InspectorApplicationCacheBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorApplicationCacheBackendDispatcher> protect(*this);

    if (method == "getFramesWithManifests")
        getFramesWithManifests(callId, *message.get());
    else if (method == "enable")
        enable(callId, *message.get());
    else if (method == "getManifestForFrame")
        getManifestForFrame(callId, *message.get());
    else if (method == "getApplicationCacheForFrame")
        getApplicationCacheForFrame(callId, *message.get());
    else
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "ApplicationCache", '.', method, "' was not found"));
}

void InspectorApplicationCacheBackendDispatcher::getFramesWithManifests(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::ApplicationCache::FrameWithManifest>> out_frameIds;
    m_agent->getFramesWithManifests(&error, out_frameIds);

    if (!error.length())
        result->setArray(ASCIILiteral("frameIds"), out_frameIds);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorApplicationCacheBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorApplicationCacheBackendDispatcher::getManifestForFrame(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_frameId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("frameId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "ApplicationCache.getManifestForFrame");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_manifestURL;
    m_agent->getManifestForFrame(&error, in_frameId, &out_manifestURL);

    if (!error.length())
        result->setString(ASCIILiteral("manifestURL"), out_manifestURL);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorApplicationCacheBackendDispatcher::getApplicationCacheForFrame(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_frameId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("frameId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "ApplicationCache.getApplicationCacheForFrame");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::ApplicationCache::ApplicationCache> out_applicationCache;
    m_agent->getApplicationCacheForFrame(&error, in_frameId, out_applicationCache);

    if (!error.length())
        result->setObject(ASCIILiteral("applicationCache"), out_applicationCache);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

PassRefPtr<InspectorCSSBackendDispatcher> InspectorCSSBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorCSSBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorCSSBackendDispatcher(backendDispatcher, agent));
}

InspectorCSSBackendDispatcher::InspectorCSSBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorCSSBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("CSS"), this);
}

void InspectorCSSBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorCSSBackendDispatcher> protect(*this);

    typedef void (InspectorCSSBackendDispatcher::*CallHandler)(long callId, const Inspector::InspectorObject& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    DEPRECATED_DEFINE_STATIC_LOCAL(DispatchMap, dispatchMap, ());
    if (dispatchMap.isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "enable", &InspectorCSSBackendDispatcher::enable },
            { "disable", &InspectorCSSBackendDispatcher::disable },
            { "getMatchedStylesForNode", &InspectorCSSBackendDispatcher::getMatchedStylesForNode },
            { "getInlineStylesForNode", &InspectorCSSBackendDispatcher::getInlineStylesForNode },
            { "getComputedStyleForNode", &InspectorCSSBackendDispatcher::getComputedStyleForNode },
            { "getAllStyleSheets", &InspectorCSSBackendDispatcher::getAllStyleSheets },
            { "getStyleSheet", &InspectorCSSBackendDispatcher::getStyleSheet },
            { "getStyleSheetText", &InspectorCSSBackendDispatcher::getStyleSheetText },
            { "setStyleSheetText", &InspectorCSSBackendDispatcher::setStyleSheetText },
            { "setStyleText", &InspectorCSSBackendDispatcher::setStyleText },
            { "setPropertyText", &InspectorCSSBackendDispatcher::setPropertyText },
            { "toggleProperty", &InspectorCSSBackendDispatcher::toggleProperty },
            { "setRuleSelector", &InspectorCSSBackendDispatcher::setRuleSelector },
            { "addRule", &InspectorCSSBackendDispatcher::addRule },
            { "getSupportedCSSProperties", &InspectorCSSBackendDispatcher::getSupportedCSSProperties },
            { "forcePseudoState", &InspectorCSSBackendDispatcher::forcePseudoState },
            { "getNamedFlowCollection", &InspectorCSSBackendDispatcher::getNamedFlowCollection },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.add(commands[i].name, commands[i].handler);
    }

    HashMap<String, CallHandler>::iterator it = dispatchMap.find(method);
    if (it == dispatchMap.end()) {
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "CSS", '.', method, "' was not found"));
        return;
    }

    ((*this).*it->value)(callId, *message.get());
}

void InspectorCSSBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getMatchedStylesForNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    bool includePseudo_valueFound = false;
    bool in_includePseudo = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("includePseudo"), &includePseudo_valueFound, protocolErrorsPtr);
    bool includeInherited_valueFound = false;
    bool in_includeInherited = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("includeInherited"), &includeInherited_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.getMatchedStylesForNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::CSS::RuleMatch>> out_matchedCSSRules;
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::CSS::PseudoIdMatches>> out_pseudoElements;
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::CSS::InheritedStyleEntry>> out_inherited;
    m_agent->getMatchedStylesForNode(&error, in_nodeId, includePseudo_valueFound ? &in_includePseudo : nullptr, includeInherited_valueFound ? &in_includeInherited : nullptr, out_matchedCSSRules, out_pseudoElements, out_inherited);

    if (!error.length()) {
        if (out_matchedCSSRules)
            result->setArray(ASCIILiteral("matchedCSSRules"), out_matchedCSSRules);
        if (out_pseudoElements)
            result->setArray(ASCIILiteral("pseudoElements"), out_pseudoElements);
        if (out_inherited)
            result->setArray(ASCIILiteral("inherited"), out_inherited);
    }
    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getInlineStylesForNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.getInlineStylesForNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::CSS::CSSStyle> out_inlineStyle;
    RefPtr<Inspector::Protocol::CSS::CSSStyle> out_attributesStyle;
    m_agent->getInlineStylesForNode(&error, in_nodeId, out_inlineStyle, out_attributesStyle);

    if (!error.length()) {
        if (out_inlineStyle)
            result->setObject(ASCIILiteral("inlineStyle"), out_inlineStyle);
        if (out_attributesStyle)
            result->setObject(ASCIILiteral("attributesStyle"), out_attributesStyle);
    }
    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getComputedStyleForNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.getComputedStyleForNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::CSS::CSSComputedStyleProperty>> out_computedStyle;
    m_agent->getComputedStyleForNode(&error, in_nodeId, out_computedStyle);

    if (!error.length())
        result->setArray(ASCIILiteral("computedStyle"), out_computedStyle);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getAllStyleSheets(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::CSS::CSSStyleSheetHeader>> out_headers;
    m_agent->getAllStyleSheets(&error, out_headers);

    if (!error.length())
        result->setArray(ASCIILiteral("headers"), out_headers);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getStyleSheet(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_styleSheetId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("styleSheetId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.getStyleSheet");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::CSS::CSSStyleSheetBody> out_styleSheet;
    m_agent->getStyleSheet(&error, in_styleSheetId, out_styleSheet);

    if (!error.length())
        result->setObject(ASCIILiteral("styleSheet"), out_styleSheet);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getStyleSheetText(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_styleSheetId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("styleSheetId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.getStyleSheetText");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_text;
    m_agent->getStyleSheetText(&error, in_styleSheetId, &out_text);

    if (!error.length())
        result->setString(ASCIILiteral("text"), out_text);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::setStyleSheetText(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_styleSheetId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("styleSheetId"), nullptr, protocolErrorsPtr);
    String in_text = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("text"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.setStyleSheetText");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setStyleSheetText(&error, in_styleSheetId, in_text);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::setStyleText(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_styleId = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("styleId"), nullptr, protocolErrorsPtr);
    String in_text = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("text"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.setStyleText");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::CSS::CSSStyle> out_style;
    m_agent->setStyleText(&error, in_styleId, in_text, out_style);

    if (!error.length())
        result->setObject(ASCIILiteral("style"), out_style);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::setPropertyText(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_styleId = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("styleId"), nullptr, protocolErrorsPtr);
    int in_propertyIndex = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("propertyIndex"), nullptr, protocolErrorsPtr);
    String in_text = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("text"), nullptr, protocolErrorsPtr);
    bool in_overwrite = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("overwrite"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.setPropertyText");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::CSS::CSSStyle> out_style;
    m_agent->setPropertyText(&error, in_styleId, in_propertyIndex, in_text, in_overwrite, out_style);

    if (!error.length())
        result->setObject(ASCIILiteral("style"), out_style);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::toggleProperty(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_styleId = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("styleId"), nullptr, protocolErrorsPtr);
    int in_propertyIndex = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("propertyIndex"), nullptr, protocolErrorsPtr);
    bool in_disable = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("disable"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.toggleProperty");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::CSS::CSSStyle> out_style;
    m_agent->toggleProperty(&error, in_styleId, in_propertyIndex, in_disable, out_style);

    if (!error.length())
        result->setObject(ASCIILiteral("style"), out_style);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::setRuleSelector(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_ruleId = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("ruleId"), nullptr, protocolErrorsPtr);
    String in_selector = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("selector"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.setRuleSelector");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::CSS::CSSRule> out_rule;
    m_agent->setRuleSelector(&error, in_ruleId, in_selector, out_rule);

    if (!error.length())
        result->setObject(ASCIILiteral("rule"), out_rule);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::addRule(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_contextNodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("contextNodeId"), nullptr, protocolErrorsPtr);
    String in_selector = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("selector"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.addRule");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::CSS::CSSRule> out_rule;
    m_agent->addRule(&error, in_contextNodeId, in_selector, out_rule);

    if (!error.length())
        result->setObject(ASCIILiteral("rule"), out_rule);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getSupportedCSSProperties(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::CSS::CSSPropertyInfo>> out_cssProperties;
    m_agent->getSupportedCSSProperties(&error, out_cssProperties);

    if (!error.length())
        result->setArray(ASCIILiteral("cssProperties"), out_cssProperties);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::forcePseudoState(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    RefPtr<Inspector::InspectorArray> in_forcedPseudoClasses = InspectorBackendDispatcher::getArray(paramsContainerPtr, ASCIILiteral("forcedPseudoClasses"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.forcePseudoState");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->forcePseudoState(&error, in_nodeId, in_forcedPseudoClasses);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorCSSBackendDispatcher::getNamedFlowCollection(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_documentNodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("documentNodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "CSS.getNamedFlowCollection");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::CSS::NamedFlow>> out_namedFlows;
    m_agent->getNamedFlowCollection(&error, in_documentNodeId, out_namedFlows);

    if (!error.length())
        result->setArray(ASCIILiteral("namedFlows"), out_namedFlows);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

PassRefPtr<InspectorDOMBackendDispatcher> InspectorDOMBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorDOMBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorDOMBackendDispatcher(backendDispatcher, agent));
}

InspectorDOMBackendDispatcher::InspectorDOMBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorDOMBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("DOM"), this);
}

void InspectorDOMBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorDOMBackendDispatcher> protect(*this);

    typedef void (InspectorDOMBackendDispatcher::*CallHandler)(long callId, const Inspector::InspectorObject& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    DEPRECATED_DEFINE_STATIC_LOCAL(DispatchMap, dispatchMap, ());
    if (dispatchMap.isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "getDocument", &InspectorDOMBackendDispatcher::getDocument },
            { "requestChildNodes", &InspectorDOMBackendDispatcher::requestChildNodes },
            { "querySelector", &InspectorDOMBackendDispatcher::querySelector },
            { "querySelectorAll", &InspectorDOMBackendDispatcher::querySelectorAll },
            { "setNodeName", &InspectorDOMBackendDispatcher::setNodeName },
            { "setNodeValue", &InspectorDOMBackendDispatcher::setNodeValue },
            { "removeNode", &InspectorDOMBackendDispatcher::removeNode },
            { "setAttributeValue", &InspectorDOMBackendDispatcher::setAttributeValue },
            { "setAttributesAsText", &InspectorDOMBackendDispatcher::setAttributesAsText },
            { "removeAttribute", &InspectorDOMBackendDispatcher::removeAttribute },
            { "getEventListenersForNode", &InspectorDOMBackendDispatcher::getEventListenersForNode },
            { "getAccessibilityPropertiesForNode", &InspectorDOMBackendDispatcher::getAccessibilityPropertiesForNode },
            { "getOuterHTML", &InspectorDOMBackendDispatcher::getOuterHTML },
            { "setOuterHTML", &InspectorDOMBackendDispatcher::setOuterHTML },
            { "performSearch", &InspectorDOMBackendDispatcher::performSearch },
            { "getSearchResults", &InspectorDOMBackendDispatcher::getSearchResults },
            { "discardSearchResults", &InspectorDOMBackendDispatcher::discardSearchResults },
            { "requestNode", &InspectorDOMBackendDispatcher::requestNode },
            { "setInspectModeEnabled", &InspectorDOMBackendDispatcher::setInspectModeEnabled },
            { "highlightRect", &InspectorDOMBackendDispatcher::highlightRect },
            { "highlightQuad", &InspectorDOMBackendDispatcher::highlightQuad },
            { "highlightNode", &InspectorDOMBackendDispatcher::highlightNode },
            { "hideHighlight", &InspectorDOMBackendDispatcher::hideHighlight },
            { "highlightFrame", &InspectorDOMBackendDispatcher::highlightFrame },
            { "pushNodeByPathToFrontend", &InspectorDOMBackendDispatcher::pushNodeByPathToFrontend },
            { "pushNodeByBackendIdToFrontend", &InspectorDOMBackendDispatcher::pushNodeByBackendIdToFrontend },
            { "releaseBackendNodeIds", &InspectorDOMBackendDispatcher::releaseBackendNodeIds },
            { "resolveNode", &InspectorDOMBackendDispatcher::resolveNode },
            { "getAttributes", &InspectorDOMBackendDispatcher::getAttributes },
            { "moveTo", &InspectorDOMBackendDispatcher::moveTo },
            { "undo", &InspectorDOMBackendDispatcher::undo },
            { "redo", &InspectorDOMBackendDispatcher::redo },
            { "markUndoableState", &InspectorDOMBackendDispatcher::markUndoableState },
            { "focus", &InspectorDOMBackendDispatcher::focus },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.add(commands[i].name, commands[i].handler);
    }

    HashMap<String, CallHandler>::iterator it = dispatchMap.find(method);
    if (it == dispatchMap.end()) {
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "DOM", '.', method, "' was not found"));
        return;
    }

    ((*this).*it->value)(callId, *message.get());
}

void InspectorDOMBackendDispatcher::getDocument(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::DOM::Node> out_root;
    m_agent->getDocument(&error, out_root);

    if (!error.length())
        result->setObject(ASCIILiteral("root"), out_root);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::requestChildNodes(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    bool depth_valueFound = false;
    int in_depth = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("depth"), &depth_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.requestChildNodes");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->requestChildNodes(&error, in_nodeId, depth_valueFound ? &in_depth : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::querySelector(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_selector = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("selector"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.querySelector");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    Inspector::Protocol::DOM::NodeId out_nodeId;
    m_agent->querySelector(&error, in_nodeId, in_selector, &out_nodeId);

    if (!error.length())
        result->setInteger(ASCIILiteral("nodeId"), out_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::querySelectorAll(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_selector = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("selector"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.querySelectorAll");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::DOM::NodeId>> out_nodeIds;
    m_agent->querySelectorAll(&error, in_nodeId, in_selector, out_nodeIds);

    if (!error.length())
        result->setArray(ASCIILiteral("nodeIds"), out_nodeIds);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::setNodeName(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_name = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("name"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.setNodeName");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    Inspector::Protocol::DOM::NodeId out_nodeId;
    m_agent->setNodeName(&error, in_nodeId, in_name, &out_nodeId);

    if (!error.length())
        result->setInteger(ASCIILiteral("nodeId"), out_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::setNodeValue(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_value = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("value"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.setNodeValue");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setNodeValue(&error, in_nodeId, in_value);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::removeNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.removeNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeNode(&error, in_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::setAttributeValue(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_name = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("name"), nullptr, protocolErrorsPtr);
    String in_value = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("value"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.setAttributeValue");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setAttributeValue(&error, in_nodeId, in_name, in_value);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::setAttributesAsText(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_text = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("text"), nullptr, protocolErrorsPtr);
    bool name_valueFound = false;
    String in_name = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("name"), &name_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.setAttributesAsText");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setAttributesAsText(&error, in_nodeId, in_text, name_valueFound ? &in_name : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::removeAttribute(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_name = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("name"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.removeAttribute");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeAttribute(&error, in_nodeId, in_name);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::getEventListenersForNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    bool objectGroup_valueFound = false;
    String in_objectGroup = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("objectGroup"), &objectGroup_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.getEventListenersForNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::DOM::EventListener>> out_listeners;
    m_agent->getEventListenersForNode(&error, in_nodeId, objectGroup_valueFound ? &in_objectGroup : nullptr, out_listeners);

    if (!error.length())
        result->setArray(ASCIILiteral("listeners"), out_listeners);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::getAccessibilityPropertiesForNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.getAccessibilityPropertiesForNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::DOM::AccessibilityProperties> out_properties;
    m_agent->getAccessibilityPropertiesForNode(&error, in_nodeId, out_properties);

    if (!error.length())
        result->setObject(ASCIILiteral("properties"), out_properties);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::getOuterHTML(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.getOuterHTML");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_outerHTML;
    m_agent->getOuterHTML(&error, in_nodeId, &out_outerHTML);

    if (!error.length())
        result->setString(ASCIILiteral("outerHTML"), out_outerHTML);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::setOuterHTML(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_outerHTML = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("outerHTML"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.setOuterHTML");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setOuterHTML(&error, in_nodeId, in_outerHTML);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::performSearch(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_query = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("query"), nullptr, protocolErrorsPtr);
    bool nodeIds_valueFound = false;
    RefPtr<Inspector::InspectorArray> in_nodeIds = InspectorBackendDispatcher::getArray(paramsContainerPtr, ASCIILiteral("nodeIds"), &nodeIds_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.performSearch");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_searchId;
    int out_resultCount;
    m_agent->performSearch(&error, in_query, nodeIds_valueFound ? &in_nodeIds : nullptr, &out_searchId, &out_resultCount);

    if (!error.length()) {
        result->setString(ASCIILiteral("searchId"), out_searchId);
        result->setInteger(ASCIILiteral("resultCount"), out_resultCount);
    }
    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::getSearchResults(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_searchId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("searchId"), nullptr, protocolErrorsPtr);
    int in_fromIndex = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("fromIndex"), nullptr, protocolErrorsPtr);
    int in_toIndex = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("toIndex"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.getSearchResults");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::DOM::NodeId>> out_nodeIds;
    m_agent->getSearchResults(&error, in_searchId, in_fromIndex, in_toIndex, out_nodeIds);

    if (!error.length())
        result->setArray(ASCIILiteral("nodeIds"), out_nodeIds);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::discardSearchResults(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_searchId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("searchId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.discardSearchResults");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->discardSearchResults(&error, in_searchId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::requestNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_objectId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("objectId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.requestNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    Inspector::Protocol::DOM::NodeId out_nodeId;
    m_agent->requestNode(&error, in_objectId, &out_nodeId);

    if (!error.length())
        result->setInteger(ASCIILiteral("nodeId"), out_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::setInspectModeEnabled(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_enabled = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("enabled"), nullptr, protocolErrorsPtr);
    bool highlightConfig_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_highlightConfig = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("highlightConfig"), &highlightConfig_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.setInspectModeEnabled");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setInspectModeEnabled(&error, in_enabled, highlightConfig_valueFound ? &in_highlightConfig : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::highlightRect(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_x = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("x"), nullptr, protocolErrorsPtr);
    int in_y = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("y"), nullptr, protocolErrorsPtr);
    int in_width = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("width"), nullptr, protocolErrorsPtr);
    int in_height = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("height"), nullptr, protocolErrorsPtr);
    bool color_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_color = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("color"), &color_valueFound, protocolErrorsPtr);
    bool outlineColor_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_outlineColor = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("outlineColor"), &outlineColor_valueFound, protocolErrorsPtr);
    bool usePageCoordinates_valueFound = false;
    bool in_usePageCoordinates = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("usePageCoordinates"), &usePageCoordinates_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.highlightRect");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->highlightRect(&error, in_x, in_y, in_width, in_height, color_valueFound ? &in_color : nullptr, outlineColor_valueFound ? &in_outlineColor : nullptr, usePageCoordinates_valueFound ? &in_usePageCoordinates : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::highlightQuad(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorArray> in_quad = InspectorBackendDispatcher::getArray(paramsContainerPtr, ASCIILiteral("quad"), nullptr, protocolErrorsPtr);
    bool color_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_color = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("color"), &color_valueFound, protocolErrorsPtr);
    bool outlineColor_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_outlineColor = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("outlineColor"), &outlineColor_valueFound, protocolErrorsPtr);
    bool usePageCoordinates_valueFound = false;
    bool in_usePageCoordinates = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("usePageCoordinates"), &usePageCoordinates_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.highlightQuad");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->highlightQuad(&error, in_quad, color_valueFound ? &in_color : nullptr, outlineColor_valueFound ? &in_outlineColor : nullptr, usePageCoordinates_valueFound ? &in_usePageCoordinates : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::highlightNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_highlightConfig = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("highlightConfig"), nullptr, protocolErrorsPtr);
    bool nodeId_valueFound = false;
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), &nodeId_valueFound, protocolErrorsPtr);
    bool objectId_valueFound = false;
    String in_objectId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("objectId"), &objectId_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.highlightNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->highlightNode(&error, in_highlightConfig, nodeId_valueFound ? &in_nodeId : nullptr, objectId_valueFound ? &in_objectId : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::hideHighlight(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->hideHighlight(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::highlightFrame(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_frameId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("frameId"), nullptr, protocolErrorsPtr);
    bool contentColor_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_contentColor = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("contentColor"), &contentColor_valueFound, protocolErrorsPtr);
    bool contentOutlineColor_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_contentOutlineColor = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("contentOutlineColor"), &contentOutlineColor_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.highlightFrame");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->highlightFrame(&error, in_frameId, contentColor_valueFound ? &in_contentColor : nullptr, contentOutlineColor_valueFound ? &in_contentOutlineColor : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::pushNodeByPathToFrontend(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_path = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("path"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.pushNodeByPathToFrontend");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    Inspector::Protocol::DOM::NodeId out_nodeId;
    m_agent->pushNodeByPathToFrontend(&error, in_path, &out_nodeId);

    if (!error.length())
        result->setInteger(ASCIILiteral("nodeId"), out_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::pushNodeByBackendIdToFrontend(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_backendNodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("backendNodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.pushNodeByBackendIdToFrontend");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    Inspector::Protocol::DOM::NodeId out_nodeId;
    m_agent->pushNodeByBackendIdToFrontend(&error, in_backendNodeId, &out_nodeId);

    if (!error.length())
        result->setInteger(ASCIILiteral("nodeId"), out_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::releaseBackendNodeIds(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_nodeGroup = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("nodeGroup"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.releaseBackendNodeIds");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->releaseBackendNodeIds(&error, in_nodeGroup);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::resolveNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    bool objectGroup_valueFound = false;
    String in_objectGroup = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("objectGroup"), &objectGroup_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.resolveNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Runtime::RemoteObject> out_object;
    m_agent->resolveNode(&error, in_nodeId, objectGroup_valueFound ? &in_objectGroup : nullptr, out_object);

    if (!error.length())
        result->setObject(ASCIILiteral("object"), out_object);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::getAttributes(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.getAttributes");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<String>> out_attributes;
    m_agent->getAttributes(&error, in_nodeId, out_attributes);

    if (!error.length())
        result->setArray(ASCIILiteral("attributes"), out_attributes);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::moveTo(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    int in_targetNodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("targetNodeId"), nullptr, protocolErrorsPtr);
    bool insertBeforeNodeId_valueFound = false;
    int in_insertBeforeNodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("insertBeforeNodeId"), &insertBeforeNodeId_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.moveTo");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    Inspector::Protocol::DOM::NodeId out_nodeId;
    m_agent->moveTo(&error, in_nodeId, in_targetNodeId, insertBeforeNodeId_valueFound ? &in_insertBeforeNodeId : nullptr, &out_nodeId);

    if (!error.length())
        result->setInteger(ASCIILiteral("nodeId"), out_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::undo(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->undo(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::redo(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->redo(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::markUndoableState(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->markUndoableState(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMBackendDispatcher::focus(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOM.focus");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->focus(&error, in_nodeId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

PassRefPtr<InspectorDOMDebuggerBackendDispatcher> InspectorDOMDebuggerBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorDOMDebuggerBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorDOMDebuggerBackendDispatcher(backendDispatcher, agent));
}

InspectorDOMDebuggerBackendDispatcher::InspectorDOMDebuggerBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorDOMDebuggerBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("DOMDebugger"), this);
}

void InspectorDOMDebuggerBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorDOMDebuggerBackendDispatcher> protect(*this);

    typedef void (InspectorDOMDebuggerBackendDispatcher::*CallHandler)(long callId, const Inspector::InspectorObject& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    DEPRECATED_DEFINE_STATIC_LOCAL(DispatchMap, dispatchMap, ());
    if (dispatchMap.isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "setDOMBreakpoint", &InspectorDOMDebuggerBackendDispatcher::setDOMBreakpoint },
            { "removeDOMBreakpoint", &InspectorDOMDebuggerBackendDispatcher::removeDOMBreakpoint },
            { "setEventListenerBreakpoint", &InspectorDOMDebuggerBackendDispatcher::setEventListenerBreakpoint },
            { "removeEventListenerBreakpoint", &InspectorDOMDebuggerBackendDispatcher::removeEventListenerBreakpoint },
            { "setInstrumentationBreakpoint", &InspectorDOMDebuggerBackendDispatcher::setInstrumentationBreakpoint },
            { "removeInstrumentationBreakpoint", &InspectorDOMDebuggerBackendDispatcher::removeInstrumentationBreakpoint },
            { "setXHRBreakpoint", &InspectorDOMDebuggerBackendDispatcher::setXHRBreakpoint },
            { "removeXHRBreakpoint", &InspectorDOMDebuggerBackendDispatcher::removeXHRBreakpoint },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.add(commands[i].name, commands[i].handler);
    }

    HashMap<String, CallHandler>::iterator it = dispatchMap.find(method);
    if (it == dispatchMap.end()) {
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "DOMDebugger", '.', method, "' was not found"));
        return;
    }

    ((*this).*it->value)(callId, *message.get());
}

void InspectorDOMDebuggerBackendDispatcher::setDOMBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_type = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("type"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.setDOMBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setDOMBreakpoint(&error, in_nodeId, in_type);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMDebuggerBackendDispatcher::removeDOMBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    String in_type = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("type"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.removeDOMBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeDOMBreakpoint(&error, in_nodeId, in_type);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMDebuggerBackendDispatcher::setEventListenerBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_eventName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("eventName"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.setEventListenerBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setEventListenerBreakpoint(&error, in_eventName);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMDebuggerBackendDispatcher::removeEventListenerBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_eventName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("eventName"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.removeEventListenerBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeEventListenerBreakpoint(&error, in_eventName);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMDebuggerBackendDispatcher::setInstrumentationBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_eventName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("eventName"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.setInstrumentationBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setInstrumentationBreakpoint(&error, in_eventName);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMDebuggerBackendDispatcher::removeInstrumentationBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_eventName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("eventName"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.removeInstrumentationBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeInstrumentationBreakpoint(&error, in_eventName);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMDebuggerBackendDispatcher::setXHRBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_url = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("url"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.setXHRBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setXHRBreakpoint(&error, in_url);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMDebuggerBackendDispatcher::removeXHRBreakpoint(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_url = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("url"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMDebugger.removeXHRBreakpoint");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeXHRBreakpoint(&error, in_url);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

PassRefPtr<InspectorDOMStorageBackendDispatcher> InspectorDOMStorageBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorDOMStorageBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorDOMStorageBackendDispatcher(backendDispatcher, agent));
}

InspectorDOMStorageBackendDispatcher::InspectorDOMStorageBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorDOMStorageBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("DOMStorage"), this);
}

void InspectorDOMStorageBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorDOMStorageBackendDispatcher> protect(*this);

    if (method == "enable")
        enable(callId, *message.get());
    else if (method == "disable")
        disable(callId, *message.get());
    else if (method == "getDOMStorageItems")
        getDOMStorageItems(callId, *message.get());
    else if (method == "setDOMStorageItem")
        setDOMStorageItem(callId, *message.get());
    else if (method == "removeDOMStorageItem")
        removeDOMStorageItem(callId, *message.get());
    else
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "DOMStorage", '.', method, "' was not found"));
}

void InspectorDOMStorageBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMStorageBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMStorageBackendDispatcher::getDOMStorageItems(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_storageId = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("storageId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMStorage.getDOMStorageItems");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::Array<String>>> out_entries;
    m_agent->getDOMStorageItems(&error, in_storageId, out_entries);

    if (!error.length())
        result->setArray(ASCIILiteral("entries"), out_entries);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMStorageBackendDispatcher::setDOMStorageItem(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_storageId = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("storageId"), nullptr, protocolErrorsPtr);
    String in_key = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("key"), nullptr, protocolErrorsPtr);
    String in_value = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("value"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMStorage.setDOMStorageItem");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setDOMStorageItem(&error, in_storageId, in_key, in_value);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDOMStorageBackendDispatcher::removeDOMStorageItem(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_storageId = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("storageId"), nullptr, protocolErrorsPtr);
    String in_key = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("key"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "DOMStorage.removeDOMStorageItem");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeDOMStorageItem(&error, in_storageId, in_key);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

#if ENABLE(SQL_DATABASE)
PassRefPtr<InspectorDatabaseBackendDispatcher> InspectorDatabaseBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorDatabaseBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorDatabaseBackendDispatcher(backendDispatcher, agent));
}

InspectorDatabaseBackendDispatcher::InspectorDatabaseBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorDatabaseBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("Database"), this);
}

void InspectorDatabaseBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorDatabaseBackendDispatcher> protect(*this);

    if (method == "enable")
        enable(callId, *message.get());
    else if (method == "disable")
        disable(callId, *message.get());
    else if (method == "getDatabaseTableNames")
        getDatabaseTableNames(callId, *message.get());
    else if (method == "executeSQL")
        executeSQL(callId, *message.get());
    else
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "Database", '.', method, "' was not found"));
}

void InspectorDatabaseBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDatabaseBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorDatabaseBackendDispatcher::getDatabaseTableNames(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_databaseId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("databaseId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Database.getDatabaseTableNames");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<String>> out_tableNames;
    m_agent->getDatabaseTableNames(&error, in_databaseId, out_tableNames);

    if (!error.length())
        result->setArray(ASCIILiteral("tableNames"), out_tableNames);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

InspectorDatabaseBackendDispatcherHandler::ExecuteSQLCallback::ExecuteSQLCallback(PassRefPtr<InspectorBackendDispatcher> backendDispatcher, int id) : Inspector::InspectorBackendDispatcher::CallbackBase(backendDispatcher, id) { }

void InspectorDatabaseBackendDispatcherHandler::ExecuteSQLCallback::sendSuccess(PassRefPtr<Inspector::Protocol::Array<String>> columnNames, PassRefPtr<Inspector::Protocol::Array<Inspector::InspectorValue>> values, PassRefPtr<Inspector::Protocol::Database::Error> sqlError)
{
    RefPtr<InspectorObject> jsonMessage = InspectorObject::create();
    if (columnNames)
        jsonMessage->setArray(ASCIILiteral("columnNames"), columnNames);
    if (values)
        jsonMessage->setArray(ASCIILiteral("values"), values);
    if (sqlError)
        jsonMessage->setObject(ASCIILiteral("sqlError"), sqlError);
    sendIfActive(jsonMessage, ErrorString());
}

void InspectorDatabaseBackendDispatcher::executeSQL(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_databaseId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("databaseId"), nullptr, protocolErrorsPtr);
    String in_query = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("query"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Database.executeSQL");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<InspectorDatabaseBackendDispatcherHandler::ExecuteSQLCallback> callback = adoptRef(new InspectorDatabaseBackendDispatcherHandler::ExecuteSQLCallback(m_backendDispatcher,callId));
    m_agent->executeSQL(&error, in_databaseId, in_query, callback);

    if (error.length()) {
        callback->disable();
        m_backendDispatcher->reportProtocolError(&callId, Inspector::InspectorBackendDispatcher::ServerError, error);
        return;
    }
}
#endif // ENABLE(SQL_DATABASE)

#if ENABLE(INDEXED_DATABASE)
PassRefPtr<InspectorIndexedDBBackendDispatcher> InspectorIndexedDBBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorIndexedDBBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorIndexedDBBackendDispatcher(backendDispatcher, agent));
}

InspectorIndexedDBBackendDispatcher::InspectorIndexedDBBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorIndexedDBBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("IndexedDB"), this);
}

void InspectorIndexedDBBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorIndexedDBBackendDispatcher> protect(*this);

    typedef void (InspectorIndexedDBBackendDispatcher::*CallHandler)(long callId, const Inspector::InspectorObject& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    DEPRECATED_DEFINE_STATIC_LOCAL(DispatchMap, dispatchMap, ());
    if (dispatchMap.isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "enable", &InspectorIndexedDBBackendDispatcher::enable },
            { "disable", &InspectorIndexedDBBackendDispatcher::disable },
            { "requestDatabaseNames", &InspectorIndexedDBBackendDispatcher::requestDatabaseNames },
            { "requestDatabase", &InspectorIndexedDBBackendDispatcher::requestDatabase },
            { "requestData", &InspectorIndexedDBBackendDispatcher::requestData },
            { "clearObjectStore", &InspectorIndexedDBBackendDispatcher::clearObjectStore },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.add(commands[i].name, commands[i].handler);
    }

    HashMap<String, CallHandler>::iterator it = dispatchMap.find(method);
    if (it == dispatchMap.end()) {
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "IndexedDB", '.', method, "' was not found"));
        return;
    }

    ((*this).*it->value)(callId, *message.get());
}

void InspectorIndexedDBBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorIndexedDBBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseNamesCallback::RequestDatabaseNamesCallback(PassRefPtr<InspectorBackendDispatcher> backendDispatcher, int id) : Inspector::InspectorBackendDispatcher::CallbackBase(backendDispatcher, id) { }

void InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseNamesCallback::sendSuccess(PassRefPtr<Inspector::Protocol::Array<String>> databaseNames)
{
    RefPtr<InspectorObject> jsonMessage = InspectorObject::create();
    jsonMessage->setArray(ASCIILiteral("databaseNames"), databaseNames);
    sendIfActive(jsonMessage, ErrorString());
}

void InspectorIndexedDBBackendDispatcher::requestDatabaseNames(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_securityOrigin = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("securityOrigin"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "IndexedDB.requestDatabaseNames");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseNamesCallback> callback = adoptRef(new InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseNamesCallback(m_backendDispatcher,callId));
    m_agent->requestDatabaseNames(&error, in_securityOrigin, callback);

    if (error.length()) {
        callback->disable();
        m_backendDispatcher->reportProtocolError(&callId, Inspector::InspectorBackendDispatcher::ServerError, error);
        return;
    }
}

InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseCallback::RequestDatabaseCallback(PassRefPtr<InspectorBackendDispatcher> backendDispatcher, int id) : Inspector::InspectorBackendDispatcher::CallbackBase(backendDispatcher, id) { }

void InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseCallback::sendSuccess(PassRefPtr<Inspector::Protocol::IndexedDB::DatabaseWithObjectStores> databaseWithObjectStores)
{
    RefPtr<InspectorObject> jsonMessage = InspectorObject::create();
    jsonMessage->setObject(ASCIILiteral("databaseWithObjectStores"), databaseWithObjectStores);
    sendIfActive(jsonMessage, ErrorString());
}

void InspectorIndexedDBBackendDispatcher::requestDatabase(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_securityOrigin = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("securityOrigin"), nullptr, protocolErrorsPtr);
    String in_databaseName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("databaseName"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "IndexedDB.requestDatabase");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseCallback> callback = adoptRef(new InspectorIndexedDBBackendDispatcherHandler::RequestDatabaseCallback(m_backendDispatcher,callId));
    m_agent->requestDatabase(&error, in_securityOrigin, in_databaseName, callback);

    if (error.length()) {
        callback->disable();
        m_backendDispatcher->reportProtocolError(&callId, Inspector::InspectorBackendDispatcher::ServerError, error);
        return;
    }
}

InspectorIndexedDBBackendDispatcherHandler::RequestDataCallback::RequestDataCallback(PassRefPtr<InspectorBackendDispatcher> backendDispatcher, int id) : Inspector::InspectorBackendDispatcher::CallbackBase(backendDispatcher, id) { }

void InspectorIndexedDBBackendDispatcherHandler::RequestDataCallback::sendSuccess(PassRefPtr<Inspector::Protocol::Array<Inspector::Protocol::IndexedDB::DataEntry>> objectStoreDataEntries, bool hasMore)
{
    RefPtr<InspectorObject> jsonMessage = InspectorObject::create();
    jsonMessage->setArray(ASCIILiteral("objectStoreDataEntries"), objectStoreDataEntries);
    jsonMessage->setBoolean(ASCIILiteral("hasMore"), hasMore);
    sendIfActive(jsonMessage, ErrorString());
}

void InspectorIndexedDBBackendDispatcher::requestData(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_securityOrigin = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("securityOrigin"), nullptr, protocolErrorsPtr);
    String in_databaseName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("databaseName"), nullptr, protocolErrorsPtr);
    String in_objectStoreName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("objectStoreName"), nullptr, protocolErrorsPtr);
    String in_indexName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("indexName"), nullptr, protocolErrorsPtr);
    int in_skipCount = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("skipCount"), nullptr, protocolErrorsPtr);
    int in_pageSize = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("pageSize"), nullptr, protocolErrorsPtr);
    bool keyRange_valueFound = false;
    RefPtr<Inspector::InspectorObject> in_keyRange = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("keyRange"), &keyRange_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "IndexedDB.requestData");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<InspectorIndexedDBBackendDispatcherHandler::RequestDataCallback> callback = adoptRef(new InspectorIndexedDBBackendDispatcherHandler::RequestDataCallback(m_backendDispatcher,callId));
    m_agent->requestData(&error, in_securityOrigin, in_databaseName, in_objectStoreName, in_indexName, in_skipCount, in_pageSize, keyRange_valueFound ? &in_keyRange : nullptr, callback);

    if (error.length()) {
        callback->disable();
        m_backendDispatcher->reportProtocolError(&callId, Inspector::InspectorBackendDispatcher::ServerError, error);
        return;
    }
}

InspectorIndexedDBBackendDispatcherHandler::ClearObjectStoreCallback::ClearObjectStoreCallback(PassRefPtr<InspectorBackendDispatcher> backendDispatcher, int id) : Inspector::InspectorBackendDispatcher::CallbackBase(backendDispatcher, id) { }

void InspectorIndexedDBBackendDispatcherHandler::ClearObjectStoreCallback::sendSuccess()
{
    RefPtr<InspectorObject> jsonMessage = InspectorObject::create();

    sendIfActive(jsonMessage, ErrorString());
}

void InspectorIndexedDBBackendDispatcher::clearObjectStore(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_securityOrigin = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("securityOrigin"), nullptr, protocolErrorsPtr);
    String in_databaseName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("databaseName"), nullptr, protocolErrorsPtr);
    String in_objectStoreName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("objectStoreName"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "IndexedDB.clearObjectStore");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<InspectorIndexedDBBackendDispatcherHandler::ClearObjectStoreCallback> callback = adoptRef(new InspectorIndexedDBBackendDispatcherHandler::ClearObjectStoreCallback(m_backendDispatcher,callId));
    m_agent->clearObjectStore(&error, in_securityOrigin, in_databaseName, in_objectStoreName, callback);

    if (error.length()) {
        callback->disable();
        m_backendDispatcher->reportProtocolError(&callId, Inspector::InspectorBackendDispatcher::ServerError, error);
        return;
    }
}
#endif // ENABLE(INDEXED_DATABASE)

PassRefPtr<InspectorLayerTreeBackendDispatcher> InspectorLayerTreeBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorLayerTreeBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorLayerTreeBackendDispatcher(backendDispatcher, agent));
}

InspectorLayerTreeBackendDispatcher::InspectorLayerTreeBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorLayerTreeBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("LayerTree"), this);
}

void InspectorLayerTreeBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorLayerTreeBackendDispatcher> protect(*this);

    if (method == "enable")
        enable(callId, *message.get());
    else if (method == "disable")
        disable(callId, *message.get());
    else if (method == "layersForNode")
        layersForNode(callId, *message.get());
    else if (method == "reasonsForCompositingLayer")
        reasonsForCompositingLayer(callId, *message.get());
    else
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "LayerTree", '.', method, "' was not found"));
}

void InspectorLayerTreeBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorLayerTreeBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorLayerTreeBackendDispatcher::layersForNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "LayerTree.layersForNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::LayerTree::Layer>> out_layers;
    m_agent->layersForNode(&error, in_nodeId, out_layers);

    if (!error.length())
        result->setArray(ASCIILiteral("layers"), out_layers);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorLayerTreeBackendDispatcher::reasonsForCompositingLayer(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_layerId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("layerId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "LayerTree.reasonsForCompositingLayer");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::LayerTree::CompositingReasons> out_compositingReasons;
    m_agent->reasonsForCompositingLayer(&error, in_layerId, out_compositingReasons);

    if (!error.length())
        result->setObject(ASCIILiteral("compositingReasons"), out_compositingReasons);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

PassRefPtr<InspectorNetworkBackendDispatcher> InspectorNetworkBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorNetworkBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorNetworkBackendDispatcher(backendDispatcher, agent));
}

InspectorNetworkBackendDispatcher::InspectorNetworkBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorNetworkBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("Network"), this);
}

void InspectorNetworkBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorNetworkBackendDispatcher> protect(*this);

    typedef void (InspectorNetworkBackendDispatcher::*CallHandler)(long callId, const Inspector::InspectorObject& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    DEPRECATED_DEFINE_STATIC_LOCAL(DispatchMap, dispatchMap, ());
    if (dispatchMap.isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "enable", &InspectorNetworkBackendDispatcher::enable },
            { "disable", &InspectorNetworkBackendDispatcher::disable },
            { "setExtraHTTPHeaders", &InspectorNetworkBackendDispatcher::setExtraHTTPHeaders },
            { "getResponseBody", &InspectorNetworkBackendDispatcher::getResponseBody },
            { "replayXHR", &InspectorNetworkBackendDispatcher::replayXHR },
            { "canClearBrowserCache", &InspectorNetworkBackendDispatcher::canClearBrowserCache },
            { "clearBrowserCache", &InspectorNetworkBackendDispatcher::clearBrowserCache },
            { "canClearBrowserCookies", &InspectorNetworkBackendDispatcher::canClearBrowserCookies },
            { "clearBrowserCookies", &InspectorNetworkBackendDispatcher::clearBrowserCookies },
            { "setCacheDisabled", &InspectorNetworkBackendDispatcher::setCacheDisabled },
            { "loadResource", &InspectorNetworkBackendDispatcher::loadResource },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.add(commands[i].name, commands[i].handler);
    }

    HashMap<String, CallHandler>::iterator it = dispatchMap.find(method);
    if (it == dispatchMap.end()) {
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "Network", '.', method, "' was not found"));
        return;
    }

    ((*this).*it->value)(callId, *message.get());
}

void InspectorNetworkBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::setExtraHTTPHeaders(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    RefPtr<Inspector::InspectorObject> in_headers = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("headers"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Network.setExtraHTTPHeaders");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setExtraHTTPHeaders(&error, in_headers);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::getResponseBody(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_requestId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("requestId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Network.getResponseBody");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_body;
    bool out_base64Encoded;
    m_agent->getResponseBody(&error, in_requestId, &out_body, &out_base64Encoded);

    if (!error.length()) {
        result->setString(ASCIILiteral("body"), out_body);
        result->setBoolean(ASCIILiteral("base64Encoded"), out_base64Encoded);
    }
    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::replayXHR(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_requestId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("requestId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Network.replayXHR");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->replayXHR(&error, in_requestId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::canClearBrowserCache(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    bool out_result;
    m_agent->canClearBrowserCache(&error, &out_result);

    if (!error.length())
        result->setBoolean(ASCIILiteral("result"), out_result);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::clearBrowserCache(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->clearBrowserCache(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::canClearBrowserCookies(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    bool out_result;
    m_agent->canClearBrowserCookies(&error, &out_result);

    if (!error.length())
        result->setBoolean(ASCIILiteral("result"), out_result);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::clearBrowserCookies(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->clearBrowserCookies(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorNetworkBackendDispatcher::setCacheDisabled(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_cacheDisabled = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("cacheDisabled"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Network.setCacheDisabled");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setCacheDisabled(&error, in_cacheDisabled);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

InspectorNetworkBackendDispatcherHandler::LoadResourceCallback::LoadResourceCallback(PassRefPtr<InspectorBackendDispatcher> backendDispatcher, int id) : Inspector::InspectorBackendDispatcher::CallbackBase(backendDispatcher, id) { }

void InspectorNetworkBackendDispatcherHandler::LoadResourceCallback::sendSuccess(const String& content, const String& mimeType, double status)
{
    RefPtr<InspectorObject> jsonMessage = InspectorObject::create();
    jsonMessage->setString(ASCIILiteral("content"), content);
    jsonMessage->setString(ASCIILiteral("mimeType"), mimeType);
    jsonMessage->setDouble(ASCIILiteral("status"), status);
    sendIfActive(jsonMessage, ErrorString());
}

void InspectorNetworkBackendDispatcher::loadResource(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_frameId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("frameId"), nullptr, protocolErrorsPtr);
    String in_url = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("url"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Network.loadResource");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<InspectorNetworkBackendDispatcherHandler::LoadResourceCallback> callback = adoptRef(new InspectorNetworkBackendDispatcherHandler::LoadResourceCallback(m_backendDispatcher,callId));
    m_agent->loadResource(&error, in_frameId, in_url, callback);

    if (error.length()) {
        callback->disable();
        m_backendDispatcher->reportProtocolError(&callId, Inspector::InspectorBackendDispatcher::ServerError, error);
        return;
    }
}

PassRefPtr<InspectorPageBackendDispatcher> InspectorPageBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorPageBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorPageBackendDispatcher(backendDispatcher, agent));
}

InspectorPageBackendDispatcher::InspectorPageBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorPageBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("Page"), this);
}

void InspectorPageBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorPageBackendDispatcher> protect(*this);

    typedef void (InspectorPageBackendDispatcher::*CallHandler)(long callId, const Inspector::InspectorObject& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    DEPRECATED_DEFINE_STATIC_LOCAL(DispatchMap, dispatchMap, ());
    if (dispatchMap.isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "enable", &InspectorPageBackendDispatcher::enable },
            { "disable", &InspectorPageBackendDispatcher::disable },
            { "addScriptToEvaluateOnLoad", &InspectorPageBackendDispatcher::addScriptToEvaluateOnLoad },
            { "removeScriptToEvaluateOnLoad", &InspectorPageBackendDispatcher::removeScriptToEvaluateOnLoad },
            { "reload", &InspectorPageBackendDispatcher::reload },
            { "navigate", &InspectorPageBackendDispatcher::navigate },
            { "getCookies", &InspectorPageBackendDispatcher::getCookies },
            { "deleteCookie", &InspectorPageBackendDispatcher::deleteCookie },
            { "getResourceTree", &InspectorPageBackendDispatcher::getResourceTree },
            { "getResourceContent", &InspectorPageBackendDispatcher::getResourceContent },
            { "searchInResource", &InspectorPageBackendDispatcher::searchInResource },
            { "searchInResources", &InspectorPageBackendDispatcher::searchInResources },
            { "setDocumentContent", &InspectorPageBackendDispatcher::setDocumentContent },
            { "setShowPaintRects", &InspectorPageBackendDispatcher::setShowPaintRects },
            { "canShowDebugBorders", &InspectorPageBackendDispatcher::canShowDebugBorders },
            { "setShowDebugBorders", &InspectorPageBackendDispatcher::setShowDebugBorders },
            { "canShowFPSCounter", &InspectorPageBackendDispatcher::canShowFPSCounter },
            { "setShowFPSCounter", &InspectorPageBackendDispatcher::setShowFPSCounter },
            { "canContinuouslyPaint", &InspectorPageBackendDispatcher::canContinuouslyPaint },
            { "setContinuousPaintingEnabled", &InspectorPageBackendDispatcher::setContinuousPaintingEnabled },
            { "getScriptExecutionStatus", &InspectorPageBackendDispatcher::getScriptExecutionStatus },
            { "setScriptExecutionDisabled", &InspectorPageBackendDispatcher::setScriptExecutionDisabled },
            { "setTouchEmulationEnabled", &InspectorPageBackendDispatcher::setTouchEmulationEnabled },
            { "setEmulatedMedia", &InspectorPageBackendDispatcher::setEmulatedMedia },
            { "getCompositingBordersVisible", &InspectorPageBackendDispatcher::getCompositingBordersVisible },
            { "setCompositingBordersVisible", &InspectorPageBackendDispatcher::setCompositingBordersVisible },
            { "snapshotNode", &InspectorPageBackendDispatcher::snapshotNode },
            { "snapshotRect", &InspectorPageBackendDispatcher::snapshotRect },
            { "handleJavaScriptDialog", &InspectorPageBackendDispatcher::handleJavaScriptDialog },
            { "archive", &InspectorPageBackendDispatcher::archive },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.add(commands[i].name, commands[i].handler);
    }

    HashMap<String, CallHandler>::iterator it = dispatchMap.find(method);
    if (it == dispatchMap.end()) {
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "Page", '.', method, "' was not found"));
        return;
    }

    ((*this).*it->value)(callId, *message.get());
}

void InspectorPageBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::addScriptToEvaluateOnLoad(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_scriptSource = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("scriptSource"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.addScriptToEvaluateOnLoad");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    Inspector::Protocol::Page::ScriptIdentifier out_identifier;
    m_agent->addScriptToEvaluateOnLoad(&error, in_scriptSource, &out_identifier);

    if (!error.length())
        result->setString(ASCIILiteral("identifier"), out_identifier);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::removeScriptToEvaluateOnLoad(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_identifier = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("identifier"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.removeScriptToEvaluateOnLoad");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->removeScriptToEvaluateOnLoad(&error, in_identifier);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::reload(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool ignoreCache_valueFound = false;
    bool in_ignoreCache = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("ignoreCache"), &ignoreCache_valueFound, protocolErrorsPtr);
    bool scriptToEvaluateOnLoad_valueFound = false;
    String in_scriptToEvaluateOnLoad = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("scriptToEvaluateOnLoad"), &scriptToEvaluateOnLoad_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.reload");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->reload(&error, ignoreCache_valueFound ? &in_ignoreCache : nullptr, scriptToEvaluateOnLoad_valueFound ? &in_scriptToEvaluateOnLoad : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::navigate(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_url = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("url"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.navigate");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->navigate(&error, in_url);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::getCookies(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::Page::Cookie>> out_cookies;
    m_agent->getCookies(&error, out_cookies);

    if (!error.length())
        result->setArray(ASCIILiteral("cookies"), out_cookies);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::deleteCookie(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_cookieName = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("cookieName"), nullptr, protocolErrorsPtr);
    String in_url = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("url"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.deleteCookie");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->deleteCookie(&error, in_cookieName, in_url);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::getResourceTree(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Page::FrameResourceTree> out_frameTree;
    m_agent->getResourceTree(&error, out_frameTree);

    if (!error.length())
        result->setObject(ASCIILiteral("frameTree"), out_frameTree);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::getResourceContent(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_frameId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("frameId"), nullptr, protocolErrorsPtr);
    String in_url = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("url"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.getResourceContent");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_content;
    bool out_base64Encoded;
    m_agent->getResourceContent(&error, in_frameId, in_url, &out_content, &out_base64Encoded);

    if (!error.length()) {
        result->setString(ASCIILiteral("content"), out_content);
        result->setBoolean(ASCIILiteral("base64Encoded"), out_base64Encoded);
    }
    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::searchInResource(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_frameId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("frameId"), nullptr, protocolErrorsPtr);
    String in_url = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("url"), nullptr, protocolErrorsPtr);
    String in_query = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("query"), nullptr, protocolErrorsPtr);
    bool caseSensitive_valueFound = false;
    bool in_caseSensitive = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("caseSensitive"), &caseSensitive_valueFound, protocolErrorsPtr);
    bool isRegex_valueFound = false;
    bool in_isRegex = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("isRegex"), &isRegex_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.searchInResource");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::GenericTypes::SearchMatch>> out_result;
    m_agent->searchInResource(&error, in_frameId, in_url, in_query, caseSensitive_valueFound ? &in_caseSensitive : nullptr, isRegex_valueFound ? &in_isRegex : nullptr, out_result);

    if (!error.length())
        result->setArray(ASCIILiteral("result"), out_result);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::searchInResources(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_text = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("text"), nullptr, protocolErrorsPtr);
    bool caseSensitive_valueFound = false;
    bool in_caseSensitive = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("caseSensitive"), &caseSensitive_valueFound, protocolErrorsPtr);
    bool isRegex_valueFound = false;
    bool in_isRegex = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("isRegex"), &isRegex_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.searchInResources");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    RefPtr<Inspector::Protocol::Array<Inspector::Protocol::Page::SearchResult>> out_result;
    m_agent->searchInResources(&error, in_text, caseSensitive_valueFound ? &in_caseSensitive : nullptr, isRegex_valueFound ? &in_isRegex : nullptr, out_result);

    if (!error.length())
        result->setArray(ASCIILiteral("result"), out_result);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setDocumentContent(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_frameId = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("frameId"), nullptr, protocolErrorsPtr);
    String in_html = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("html"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setDocumentContent");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setDocumentContent(&error, in_frameId, in_html);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setShowPaintRects(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_result = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("result"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setShowPaintRects");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setShowPaintRects(&error, in_result);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::canShowDebugBorders(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    bool out_show;
    m_agent->canShowDebugBorders(&error, &out_show);

    if (!error.length())
        result->setBoolean(ASCIILiteral("show"), out_show);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setShowDebugBorders(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_show = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("show"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setShowDebugBorders");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setShowDebugBorders(&error, in_show);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::canShowFPSCounter(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    bool out_show;
    m_agent->canShowFPSCounter(&error, &out_show);

    if (!error.length())
        result->setBoolean(ASCIILiteral("show"), out_show);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setShowFPSCounter(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_show = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("show"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setShowFPSCounter");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setShowFPSCounter(&error, in_show);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::canContinuouslyPaint(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    bool out_value;
    m_agent->canContinuouslyPaint(&error, &out_value);

    if (!error.length())
        result->setBoolean(ASCIILiteral("value"), out_value);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setContinuousPaintingEnabled(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_enabled = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("enabled"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setContinuousPaintingEnabled");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setContinuousPaintingEnabled(&error, in_enabled);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::getScriptExecutionStatus(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    InspectorPageBackendDispatcherHandler::Result out_result;
    m_agent->getScriptExecutionStatus(&error, &out_result);

    if (!error.length())
        result->setString(ASCIILiteral("result"), Inspector::Protocol::getWebEnumConstantValue(out_result));

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setScriptExecutionDisabled(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_value = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("value"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setScriptExecutionDisabled");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setScriptExecutionDisabled(&error, in_value);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setTouchEmulationEnabled(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_enabled = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("enabled"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setTouchEmulationEnabled");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setTouchEmulationEnabled(&error, in_enabled);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setEmulatedMedia(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    String in_media = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("media"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setEmulatedMedia");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setEmulatedMedia(&error, in_media);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::getCompositingBordersVisible(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    bool out_result;
    m_agent->getCompositingBordersVisible(&error, &out_result);

    if (!error.length())
        result->setBoolean(ASCIILiteral("result"), out_result);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::setCompositingBordersVisible(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_visible = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("visible"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.setCompositingBordersVisible");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setCompositingBordersVisible(&error, in_visible);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::snapshotNode(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_nodeId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("nodeId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.snapshotNode");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_dataURL;
    m_agent->snapshotNode(&error, in_nodeId, &out_dataURL);

    if (!error.length())
        result->setString(ASCIILiteral("dataURL"), out_dataURL);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::snapshotRect(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_x = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("x"), nullptr, protocolErrorsPtr);
    int in_y = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("y"), nullptr, protocolErrorsPtr);
    int in_width = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("width"), nullptr, protocolErrorsPtr);
    int in_height = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("height"), nullptr, protocolErrorsPtr);
    String in_coordinateSystem = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("coordinateSystem"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.snapshotRect");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_dataURL;
    m_agent->snapshotRect(&error, in_x, in_y, in_width, in_height, in_coordinateSystem, &out_dataURL);

    if (!error.length())
        result->setString(ASCIILiteral("dataURL"), out_dataURL);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::handleJavaScriptDialog(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_accept = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("accept"), nullptr, protocolErrorsPtr);
    bool promptText_valueFound = false;
    String in_promptText = InspectorBackendDispatcher::getString(paramsContainerPtr, ASCIILiteral("promptText"), &promptText_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Page.handleJavaScriptDialog");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->handleJavaScriptDialog(&error, in_accept, promptText_valueFound ? &in_promptText : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorPageBackendDispatcher::archive(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    String out_data;
    m_agent->archive(&error, &out_data);

    if (!error.length())
        result->setString(ASCIILiteral("data"), out_data);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

PassRefPtr<InspectorTimelineBackendDispatcher> InspectorTimelineBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorTimelineBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorTimelineBackendDispatcher(backendDispatcher, agent));
}

InspectorTimelineBackendDispatcher::InspectorTimelineBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorTimelineBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("Timeline"), this);
}

void InspectorTimelineBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorTimelineBackendDispatcher> protect(*this);

    if (method == "start")
        start(callId, *message.get());
    else if (method == "stop")
        stop(callId, *message.get());
    else
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "Timeline", '.', method, "' was not found"));
}

void InspectorTimelineBackendDispatcher::start(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool maxCallStackDepth_valueFound = false;
    int in_maxCallStackDepth = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("maxCallStackDepth"), &maxCallStackDepth_valueFound, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Timeline.start");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->start(&error, maxCallStackDepth_valueFound ? &in_maxCallStackDepth : nullptr);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorTimelineBackendDispatcher::stop(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->stop(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

PassRefPtr<InspectorWorkerBackendDispatcher> InspectorWorkerBackendDispatcher::create(InspectorBackendDispatcher* backendDispatcher, InspectorWorkerBackendDispatcherHandler* agent)
{
    return adoptRef(new InspectorWorkerBackendDispatcher(backendDispatcher, agent));
}

InspectorWorkerBackendDispatcher::InspectorWorkerBackendDispatcher(InspectorBackendDispatcher* backendDispatcher, InspectorWorkerBackendDispatcherHandler* agent)
    : InspectorSupplementalBackendDispatcher(backendDispatcher)
    , m_agent(agent)
{
    m_backendDispatcher->registerDispatcherForDomain(ASCIILiteral("Worker"), this);
}

void InspectorWorkerBackendDispatcher::dispatch(long callId, const String& method, PassRefPtr<InspectorObject> message)
{
    Ref<InspectorWorkerBackendDispatcher> protect(*this);

    typedef void (InspectorWorkerBackendDispatcher::*CallHandler)(long callId, const Inspector::InspectorObject& message);
    typedef HashMap<String, CallHandler> DispatchMap;
    DEPRECATED_DEFINE_STATIC_LOCAL(DispatchMap, dispatchMap, ());
    if (dispatchMap.isEmpty()) {
        static const struct MethodTable {
            const char* name;
            CallHandler handler;
        } commands[] = {
            { "enable", &InspectorWorkerBackendDispatcher::enable },
            { "disable", &InspectorWorkerBackendDispatcher::disable },
            { "sendMessageToWorker", &InspectorWorkerBackendDispatcher::sendMessageToWorker },
            { "canInspectWorkers", &InspectorWorkerBackendDispatcher::canInspectWorkers },
            { "connectToWorker", &InspectorWorkerBackendDispatcher::connectToWorker },
            { "disconnectFromWorker", &InspectorWorkerBackendDispatcher::disconnectFromWorker },
            { "setAutoconnectToWorkers", &InspectorWorkerBackendDispatcher::setAutoconnectToWorkers },
        };
        size_t length = WTF_ARRAY_LENGTH(commands);
        for (size_t i = 0; i < length; ++i)
            dispatchMap.add(commands[i].name, commands[i].handler);
    }

    HashMap<String, CallHandler>::iterator it = dispatchMap.find(method);
    if (it == dispatchMap.end()) {
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::MethodNotFound, makeString('\'', "Worker", '.', method, "' was not found"));
        return;
    }

    ((*this).*it->value)(callId, *message.get());
}

void InspectorWorkerBackendDispatcher::enable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->enable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorWorkerBackendDispatcher::disable(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disable(&error);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorWorkerBackendDispatcher::sendMessageToWorker(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_workerId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("workerId"), nullptr, protocolErrorsPtr);
    RefPtr<Inspector::InspectorObject> in_message = InspectorBackendDispatcher::getObject(paramsContainerPtr, ASCIILiteral("message"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Worker.sendMessageToWorker");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->sendMessageToWorker(&error, in_workerId, in_message);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorWorkerBackendDispatcher::canInspectWorkers(long callId, const InspectorObject&)
{
    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    bool out_result;
    m_agent->canInspectWorkers(&error, &out_result);

    if (!error.length())
        result->setBoolean(ASCIILiteral("result"), out_result);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorWorkerBackendDispatcher::connectToWorker(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_workerId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("workerId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Worker.connectToWorker");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->connectToWorker(&error, in_workerId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorWorkerBackendDispatcher::disconnectFromWorker(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    int in_workerId = InspectorBackendDispatcher::getInteger(paramsContainerPtr, ASCIILiteral("workerId"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Worker.disconnectFromWorker");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->disconnectFromWorker(&error, in_workerId);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

void InspectorWorkerBackendDispatcher::setAutoconnectToWorkers(long callId, const InspectorObject& message)
{
    RefPtr<InspectorArray> protocolErrors = InspectorArray::create();
    RefPtr<InspectorObject> paramsContainer = message.getObject(ASCIILiteral("params"));
    InspectorObject* paramsContainerPtr = paramsContainer.get();
    InspectorArray* protocolErrorsPtr = protocolErrors.get();
    bool in_value = InspectorBackendDispatcher::getBoolean(paramsContainerPtr, ASCIILiteral("value"), nullptr, protocolErrorsPtr);
    if (protocolErrors->length()) {
        String errorMessage = String::format("Some arguments of method '%s' can't be processed", "Worker.setAutoconnectToWorkers");
        m_backendDispatcher->reportProtocolError(&callId, InspectorBackendDispatcher::InvalidParams, errorMessage, protocolErrors.release());
        return;
    }

    ErrorString error;
    RefPtr<InspectorObject> result = InspectorObject::create();
    m_agent->setAutoconnectToWorkers(&error, in_value);

    m_backendDispatcher->sendResponse(callId, result.release(), error);
}

} // namespace Inspector

#endif // ENABLE(INSPECTOR)
